Artifact for Misconceptions in Finite-Trace and Infinite-Trace Linear Temporal Logic
===

Welcome!

This artifact contains the survey instruments, final catalog, and labeled
responses from our FM 2024 paper:

_Misconceptions in Finite-Trace and Infinite-Trace Linear Temporal Logic_

The artifact is primarily a dataset. There is no required software to run.

The instruments and catalog are PDF files.

The labeled responses are in spreadsheets, which we provide as open document
files (`.ods`) with HTML as a backup.

Additional files include unlabeled survey data (.tsv format) and
data-processing scripts (`.rkt` format). The scripts have been tested with
Racket v8.10 but are likely to run on any release back to v6.8.


#### Survey Instruments (Section 3)

* `a23/ltlf-survey.pdf`
* `a24/ltlf-survey.pdf`
* `ftai23/survey.pdf`
* `b22/survey1.pdf`
* `b22/survey2.pdf`

The `a` folders also contain LTL surveys that the students completed prior to
the LTLf survey. There are few mistakes in those results.

In `b22`, survey1 and survey2 correspond to hw1 and hw2 from the paper.


#### Final Catalog (Section 5, Section 7)

* `final-code-book.pdf`

The catalog begins with high-level instructions about how to label various
question formats.

It goes into detail for the "Write Formulas" questions with a series of entries
consisting of a label, a brief description, and example reponses.


#### Labeled Responses (Section 6)

* `a23/a23.ods`
* `a24/a24.ods`
* `ftai23/ftai23.ods`
* `b22/b22.ods`

The spreadsheets have tabs for the different rounds and question types. For
example, the b22 sheet covers the `\beta-hw1` and `\beta-hw2` data from
the paper.

The top row of each tab should match Table 3 in the paper.

Each sheet also has an "overview" tab with the number of incorrect responses.
To cross-check these numbers, in each tab check the column labeled "Ok" and the
cell labeled "Total responses". These numbers should agree with the incorrect
responses reported in Table 2 of the paper.

If you have trouble viewing these `.ods` files, there are matching `.zip` files
with HTML versions of the sheets.

##### Legend

Spreadsheet columns use the following abbreviations for catalog labels.
Some labels did not appear in the final catalog, and some changed names
during our process:

- BP = Bad Prop
- BSI = Bad State Index
- BSQ = Bad State Quantification
- CyG = Cycle G
- IF = Implicit F
- IG = Implicit G
- IP (or IPfx) = Implicit Prefix
- OI = Other Implicit
- WU = Weak Until
- EU = Exclusive Until
- TSU = Trace-Split U
- SX (or XX, SeqX) = Spreading X
- SG = Strong G
- SXw = Strong Xw
- WF = Weak F
- Len (or LenX) = Length
- Last = Last
- Inf = Unuused column. We may have intended this as a variant of Length.
- RV = Reasonable Variant
- Scope = Scope / Precedence


#### Other

The folders contain two other kinds of data:

- `*.tsv`: Anonymized response data without any of our labels
- `init.rkt`: Racket scripts for parsing raw response data. These scripts
  establish a link between the original data and the labeled spreadsheets, but
  are otherwise unimportant to the conclusions in the paper. Each folder has
  one script. Example uses follow. To apply to scripts to different
  spreadsheets in the same directory, however, the code itself needs minor
  changes, e.g., comment out the "ltlf" line at the bottom and uncomment the
  "ltl" line. (These scripts are provided as-is for completeness.)
  + `racket a23/init.rkt a23/ltlf-edit.tsv` prints 663 lines of tab-separated
    rows that can be pasted into a Google Spreadsheet, etc.
  + `racket ftai23/init.rkt ftai23/edit.tsv` prints 879 lines
  + `racket a24/init.rkt a24/ltlf-edit.tsv` prints 739 lines
  + `racket b22/init.rkt b22/hw1.tsv` prints 425 lines

