#lang racket

;; 

;; ---

(define traceq*
  '(("Q11" "Q47" "E" "{LD} {ELD}*" "No")
    ("Q62" "Q67" "E" "{ELD} {ELD} {ELD} {ELD} {}*" "Yes")
    ("Q68" "Q63" "!G(D)" "{D} {D} {}*" "Yes")
    ("Q45" "Q48" "X(X(X(E)))" "{E} {} {} {E} {}*" "Yes")
    ("Q18" "Q54" "F(G(E))" "{} {ELD} {} {ELD} {}*" "No")
    ("Q14" "Q50" "E U D" "{EL} {EL} {EL} {ELD} {L}*" "Yes")
    ("Q17" "Q51" "E U L" "{E}*" "No")
    ("Q15" "Q53" "F(E) & F(D)" "{} {D} {} {} {E}*" "Yes")
    ("Q19" "Q55" "G(E ==> L)" "{}*" "Yes")))

(define leq*
  '(("Q28" "Q57" "X(X(F(X(E))))" "E at 4th state or later")
    ("Q29" "Q58" "F(E) ==> G(L)" "if ever E, then L always from start")))

(define elq*
  '(("Q37" "Q61" "Whenever the Engine light is on, it is off in the next state and on again in the state after that." "G(E ==> X(!E) & XX(E))")
    ("Q39" "Q63" "E cannot stay on 3 states in a row" "!F(E & XE & XXE)")))

(define traceq*2
  '(("Q11" "Q47" "E" "{LD} {ELD}*" "No")
    ("Q45" "Q48" "X(X(X(E)))" "{E} {} {} {E} {}*" "Yes")
    ("Q13" "Q49" "G(E ==> XXX(E))" "{} {ELD} {ELD} {} {ELD}*" "Yes")
    ("Q14" "Q50" "E U D" "{EL} {EL} {EL} {ELD} {L}*" "Yes")
    ("Q15" "Q51" "F(E) & F(D)" "{} {D} {} {} {E}*" "Yes")
    ("Q16" "Q52" "X(X(F(E)))" "{ELD}*" "Yes")
    ("Q17" "Q53" "E U L" "{E}*" "No")
    ("Q18" "Q54" "F(G(E))" "{} {ELD} {} {ELD} {}*" "No")
    ("Q19" "Q55" "G(E ==> L)" "{}*" "Yes")))

(define leq*2
  '(("Q27" "Q56" "E ==> X(X(X(E)))" "E now implies E in 3 steps")
    ("Q28" "Q57" "X(X(F(X(E))))" "E at 4th state or later")
    ("Q29" "Q58" "F(E) ==> G(L)" "if ever E, then L always from start")
    ("Q30" "Q59" "(E U L) & G(E)" "always E eventually L")
    ("Q31" "Q60" "G(E ==> (X(!E) & XX(E)))" "Whenever E, goes on-off forevermore")))

(define elq*2
  '(("Q37" "Q61" "Whenever the engine is on, it is off in the next state and on again in the state after that." "G(E ==> X(!E) & XX(E))")
    ("Q38" "Q62" "E in exactly one state, not necessarily the first" "(!E) U (E & X(G(!E)))")
    ("Q39" "Q63" "E cannot stay on 3 states in a row" "!F(E & XE & XXE)")
    ("Q40" "Q64" "Whenever E, then L then or later" "G(E ==> F(L))")
    ("Q41" "Q65" "E for zero or more, then !E forevermore" "(E U !E) & (G(!E ==> G(!E)))")))

(define (parse tsv)
  (define line* (file->lines tsv))
  #;(printf "nlines = ~s~n" (length line*))
  (define q* (tab-split (first line*)))
  (define prompt* (tab-split (second line*)))
  (define NCOL (length q*))
  (define row**
    ;; ignore first three lines
    (for/list ([str (in-list (cdddr line*))])
      (define vv (tab-split str))
      (define nc (length vv))
      (if (= nc NCOL)
        vv
        (raise-arguments-error 'badlen "bad row" "expected" NCOL "got" nc "row" vv))
      ))
  #;(printf "looks okay?~n ~s~n ~s~n ~a~n"
          q*
          #;prompt*
          (first row**)
          (third row**))
  (list q* prompt* row**))

(define (*index-of xx x)
  (define rr* (indexes-of xx x))
  (cond
    ((null? rr*)
    (raise-arguments-error 'index-of "not found" "key" x "list" xx))
    ((null? (cdr rr*))
     (car rr*))
    (else
      ;; OH NO
      (last rr*))))

(define (trace-sat dd)
  (define-values [q* prompt* row**] (apply values dd))
  (define id-idx (*index-of q* "ResponseId"))
  (define vv
    (for/list ((rq (in-list traceq*)))
      (define ans-idx (*index-of q* (first rq)))
      (define exp-idx (*index-of q* (second rq)))
      (define formula (third rq))
      (define trace (fourth rq))
      (define expected (fifth rq))
      (for/list ((rr (in-list row**)))
        ;; Answer	Explanation	ID	Formula	Trace	Expected
        (list (list-ref rr ans-idx)
              (list-ref rr exp-idx)
              (list-ref rr id-idx)
              formula
              trace
              expected))))
  vv)

(define (ltl->eng dd)
  (get-5x dd leq*))

(define (eng->ltl dd)
  (get-5x dd elq*))

(define (get-5x dd qq*)
  ;; Answer	Explanation	ID	Formula	Expected
  (define-values [q* prompt* row**] (apply values dd))
  (define id-idx (*index-of q* "ResponseId"))
  (define vv
    (for/list ((rq (in-list qq*)))
      (define ans-idx (*index-of q* (first rq)))
      (define exp-idx (*index-of q* (second rq)))
      (define formula (third rq))
      (define expected (fourth rq))
      (for/list ((rr (in-list row**)))
        (list (list-ref rr ans-idx)
              (list-ref rr exp-idx)
              (list-ref rr id-idx)
              formula
              expected))))
  vv)

(define (tab-split str)
  (string-split str "\t"))

(define (tab-join str*)
  (string-join str* "\t"))

(define (print-sheet xx)
  (for* ((qq (in-list xx))
         (rr (in-list qq)))
    (displayln (tab-join rr))))

;; ---

(module+
  main
  (require racket/cmdline)
  (command-line
    #:args (tsv)
    (let ((dd (parse tsv)))
      (printf "Trace Sat~n")
      (print-sheet (trace-sat dd))
      (printf "~n~n")
      (printf "L -> E~n")
      (print-sheet (ltl->eng dd))
      (printf "~n~n")
      (printf "E -> L~n")
      (print-sheet (eng->ltl dd))
      (printf "~n~n")
      (void))))
