#lang racket

;; TODO explanations are all mixed up, need the first N for each response
;;  describe, write, both have 2 parts

(define ltlf-tracereject*
  '((tr "Q14" "Q15" "X(G(r))" "{r}*" "too short")
    (tr "Q16" "Q17" "r U (!r & F(b))" "{r} {r} {}" "mismatch")
    (tr "Q18" "Q19" "G(r) & Xw(Xw(!r))" "{r} {r} {r}" "too long")
    (tr "Q20" "Q21" "F(G(!r))" "{g} {r}" "mismatch")
    (tr "Q22" "Q23" "r & G(r => X(X(X(r))))" "{r} {} {} {r} {} {} {r}" "inf only")))

(define ltlf-tracematch*
  '((tm "Q31" "Q32" "G(F(r))" "{r} {} {} {r}" "Yes")
    (tm "Q33" "Q34" "r U (!r)" "{r} {r} {r}" "No")
    (tm "Q35" "Q36" "F(!X(r))" "{r} {r}" "Yes")
    (tm "Q37" "Q38" "F(X(!r))" "{r} {r}" "No")
    (tm "Q39" "Q40" "F(G(r))" "{} {} {} {r}" "Yes")
    (tm "Q41" "Q42" "F(r & F(b))" "{b} {r} {r} {b}" "Yes")))

(define ltlf-describe*
  '( ;; ltl
    (d "Q49" "" "F(r => F(!r))" "if red, must have !red in future")
    (d "Q53" "" "(r U b) & G(r)" "always r, eventually b")
    (d "Q57" "" "r & !X(b)" "r now, !b next")
    (d "Q61" "" "G(r => X(!r & X(r)))" "whenever r, off/on evermore")
    ;; ltlf
    (d "Q50" "Q51" "F(r => F(!r))" "same")
    (d "Q54" "Q55" "(r U b) & G(r)" "r for all states, b for some")
    (d "Q58" "Q59" "r & !X(b)" "r now, if next !b")
    (d "Q62" "Q63" "G(r => X(!r & X(r)))" "never r")))

(define ltlf-write*
  '( ;; ltl
    (w "Q70"   "" "whenever r, cannot be next with g" "G(r => !X(g))")
    (w "Q74"   "" "There are at least two states in which the Blue light is on" "F(b & X(F(b)))")
    (w "Q78"   "" "Green is on in the final state." "inexpressible")
    (w "Q82"  "" "Blue on in 1, off in 2, alts evermore" "b & G(b => X(!b & X(b)))")
    (w "Q86"  "" "Red is on exactly once." "!r U (r & G(!X(r)))")
    ;; ltlf
    (w "Q71" "Q72" "whenever r, cannot be next with g" "same")
    (w "Q75" "Q76" "There are at least two states in which the Blue light is on" "same")
    (w "Q79" "Q80" "Green is on in the final state." "F(g & Xw(false))")
    (w "Q83" "Q84" "Blue on in 1, off in 2, alts evermore" "b & G(b => Xw(!b & Xw(b)))")
    (w "Q87" "Q88" "Red is on exactly once." "same")))

(define ltlf-identity*
  '((i "Q94" "Q95" "F(a) == true U a" "True")
    (i "Q96" "Q97" "!X(a) == X(!a)" "False")
    (i "Q98" "Q99" "X(G(a)) == G(X(a))" "False")
    (i "Q100" "Q101" "X(a U b) == X(a) U X(b)" "True")
    (i "Q102" "Q103" "G(a) == a & X(G(a))" "False")
    (i "Q104" "Q105" "F(a) == a || X(F(a))" "True")
    (i "Q106" "Q107" "G(F(a)) == F(G(a))" "True")))


(define (parse tsv)
  (define line* (file->lines tsv))
  #;(printf "nlines = ~s~n" (length line*))
  (define q* (tab-split (first line*)))
  (define prompt* (tab-split (second line*)))
  (define NCOL (length q*))
  (define row**
    ;; ignore first three lines
    (for/list ([str (in-list (cdddr line*))])
      (define vv (tab-split str))
      (define nc (length vv))
      (if (= nc NCOL)
        vv
        (raise-arguments-error 'badlen "bad row" "expected" NCOL "got" nc "row" vv))
      ))
  #;(printf "looks okay?~n ~s~n ~s~n ~a~n"
          q*
          #;prompt*
          (first row**)
          (third row**))
  (list q* prompt* row**))

(define (*index-of xx x)
  (define rr (index-of xx x))
  (or rr (raise-arguments-error 'index-of "not found" "key" x "list" xx)))

(define (ltlf-tagged dd qq)
  (define-values [q* prompt* row**] (apply values dd))
  (define id-idx (*index-of q* "ResponseId"))
  ;; TODO lots of copy/paste below!
  (define vv
    (case (car (car qq))
      ((tr)
    (for/list ((rq (in-list qq)))
      (define ans-idx (*index-of q* (second rq)))
      (define exp-idx (*index-of q* (third rq)))
      (define formula (fourth rq))
      (define trace (fifth rq))
      (define expected (sixth rq))
      (for/list ((rr (in-list row**)))
        ;; Answer	Explanation	ID	Formula	Trace	Expected
        (list (list-ref rr ans-idx)
              (list-ref rr exp-idx)
              (list-ref rr id-idx)
              formula
              trace
              expected))))

      ((tm)
    (for/list ((rq (in-list qq)))
      (define ans-idx (*index-of q* (second rq)))
      (define exp-idx (*index-of q* (third rq)))
      (define formula (fourth rq))
      (define trace (fifth rq))
      (define expected (sixth rq))
      (for/list ((rr (in-list row**)))
        ;; Answer	Explanation	ID	Formula	Trace	Expected
        (list (list-ref rr ans-idx)
              (list-ref rr exp-idx)
              (list-ref rr id-idx)
              formula
              trace
              expected))))

      ((d)
    (for/list ((rq (in-list qq)))
      (define ans-idx (*index-of q* (second rq)))
      (define exp-idx
        (let ((ss (third rq)))
          (and (non-empty-string? ss)
               (*index-of q* ss))))
      (define formula (fourth rq))
      (define expected (fifth rq))
      (for/list ((rr (in-list row**)))
  ;; Answer	Explanation	ID	Formula	Expected
        (list (list-ref rr ans-idx)
              (if exp-idx (list-ref rr exp-idx) "")
              (list-ref rr id-idx)
              formula
              expected))))

      ((i)
    (for/list ((rq (in-list qq)))
      (define ans-idx (*index-of q* (second rq)))
      (define exp-idx (*index-of q* (third rq)))
      (define formula (fourth rq))
      (define expected (fifth rq))
      (for/list ((rr (in-list row**)))
        ;; Answer	Explanation	ID	Formula	Expected
        (list (list-ref rr ans-idx)
              (list-ref rr exp-idx)
              (list-ref rr id-idx)
              formula
              expected))))

      ((w)
    (for/list ((rq (in-list qq)))
      (define ans-idx (*index-of q* (second rq)))
      (define exp-idx
        (let ((ss (third rq)))
          (and (non-empty-string? ss)
               (*index-of q* ss))))
      (define formula (fourth rq))
      (define expected (fifth rq))
      (for/list ((rr (in-list row**)))
  ;; Answer	Explanation	ID	Formula	Expected
        (list (list-ref rr ans-idx)
              (if exp-idx (list-ref rr exp-idx) "")
              (list-ref rr id-idx)
              formula
              expected))))

      (else (raise-argument-error 'print-tagged "question-type?" dd))))
  vv)

(define (tab-split str)
  (string-split str "\t"))

(define (tab-join str*)
  (string-join str* "\t"))

(define (print-sheet xx)
  (for* ((qq (in-list xx))
         (rr (in-list qq)))
    (displayln (tab-join rr))))

(define (print-ltlf tsv)
    (let ((dd (parse tsv)))
      (for ((qq (in-list   (list ltlf-tracereject* ltlf-tracematch* ltlf-describe* ltlf-identity* ltlf-write*)))
            (name (in-list '("Reject Traces" "Match Traces" "Describe Formulas" "Check Identities" "Write Formulas"))))
        (displayln name)
        (print-sheet (ltlf-tagged dd qq))
        (newline)
        (newline))))

;; ---

(module+
  main
  (require racket/cmdline)
  (command-line
    #:args (tsv)
    (print-ltlf tsv)
    (void)))
